/* 

   ICFP 2001 Contest entry
   Chris Eagle
   
*/

#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>

#define b_tag 0x80000000
#define em_tag 0x40000000
#define i_tag 0x20000000
#define s_tag 0x10000000
#define u_mask 0xC0000000
#define u_count 0x40000000
#define size_tag 0x0F
#define r_color 0x2000000
#define g_color 0x1000000
#define b_color 0x800000
#define c_color 0x400000
#define m_color 0x200000
#define y_color 0x100000
#define k_color 0x80000
#define w_color 0x40000
#define tt_flag 0x20000
#define color_mask 0x03FC0000
#define plain_mask 0x03FDFFFF

typedef struct runTag {
   unsigned int attr;
   int tag;
   void *child;
   struct runTag *next;
   struct runtag *parent;
} Run;

typedef struct stackTag {
   struct runTag *open;
   struct stackTag *next;
} Stack;

Run root = {0, 1, 0, 0, 0};
Run *active = &root;

Stack *sp = 0;

unsigned int attr = 0;
unsigned int text = 0;

Run *pop() {
   Stack *temp = sp;
   Run *r = sp->open;
   sp = sp->next;
   free(temp);
   return r;
}

void push(Run *r) {
   Stack *node;
   node = (Stack*) malloc(sizeof(Stack));
   node->next = sp;
   node->open = r;
   sp = node;
}

void addChild(Run *parent, Run *child) {
   child->parent = parent;
   if (parent->child) {
      parent = (Run*) parent->child;
      while (parent->next) parent = parent->next;
      parent->next = child;
   }
   else {
      parent->child = child;
   }
}

void activate(Run *r) {
   addChild(active, r);
   active = r;
   push(r);
}

void optimize() {
}

void closeTag() {
   Run * r = pop();
   if (r) {
      active = active->parent;
      attr = active->attr;
   }
}

void openTag(unsigned int tag) {
   Run *child = (Run *) calloc(1, sizeof(Run));
   child->tag = tag;
   child->attr = attr;
   if (tag) {
      activate(child);
   }
   else {
      push(NULL);
   }
}

void newText(int ch) {
   Run *child = (Run *) calloc(1, sizeof(Run));
   child->tag = -1;
   child->attr = 4;
   activate(child);
   child->child = malloc(4);
   ((char*)(child->child))[0] = ch;
   ((char*)(child->child))[1] = 0;
   text = 1;
}

void append(int ch) {
   char *p1 = (char *)active->child;
   unsigned int len = 0;
   while (p1[len]) len++;
   if (len == (active->attr - 1)) {
      p1 = active->child = realloc(active->child, 2 * active->attr);
      active->attr *= 2;
   }
   p1[len++] = ch;
   p1[len] = 0;
}

void tag() {
   int ch;
   ch = getchar();
   if (ch == '/') {
      ch = getchar();
      closeTag();
      while (getchar() != '>');
      return;
   }
   switch (ch) {
      case 'B':
         if (attr & b_tag) {
            ch = 0;
         }
         else attr |= b_tag;
         break;
      case 'E':
         getchar();  //should be 'M'
         if (!(attr & s_tag)) attr ^= em_tag;
         break;
      case 'I':
         if (attr & i_tag) ch = 0;
         else attr |= i_tag;
         break;
      case 'P':
         getchar();  //should be 'L'
         attr &= plain_mask;
         break;
      case 'S':
         attr = (attr | s_tag) & ~em_tag;
         break;
      case 'T':
         getchar();  //should be 'T'
         if (attr & tt_flag) ch = 0;
         else attr |= tt_flag;
         break;
      case 'U':
         if ((attr & u_mask) != u_mask) attr += u_count;
         break;
      case '0': case '1': case '2': case '3': case '4':
      case '5': case '6': case '7': case '8': case '9':
         if ((attr & size_tag) == (ch & size_tag)) ch = 0;
         else attr = (attr & ~size_tag) | (ch & size_tag);
         break;
      case 'r':
         if (attr & r_color) ch = 0;
         else attr = (attr & ~color_mask) | r_color;
         break;
      case 'g':
         if (attr & g_color) ch = 0;
         else attr = (attr & ~color_mask) | g_color;
         break;
      case 'b':
         if (attr & b_color) ch = 0;
         else attr = (attr & ~color_mask) | b_color;
         break;
      case 'c':
         if (attr & c_color) ch = 0;
         else attr = (attr & ~color_mask) | c_color;
         break;
      case 'm':
         if (attr & m_color) ch = 0;
         else attr = (attr & ~color_mask) | m_color;
         break;
      case 'y':
         if (attr & y_color) ch = 0;
         else attr = (attr & ~color_mask) | y_color;
         break;
      case 'k':
         if (attr & k_color) ch = 0;
         else attr = (attr & ~color_mask) | k_color;
         break;
      case 'w':
         if (attr & w_color) ch = 0;
         else attr = (attr & ~color_mask) | w_color;
         break;
   }
   openTag(ch);
   getchar();  //should be a '>'
}

void dumpTree(Run *ptr) {
   if (!ptr) return;
   if (ptr->tag == -1) {
      fputs((char*) ptr->child, stdout);
      dumpTree(ptr->next);
      return;
   }
   if (ptr->tag > 1) {
      fputc('<', stdout);
      fputc(ptr->tag, stdout);
      switch (ptr->tag) {
         case 'E':
            fputc('M', stdout);
            break;
         case 'P':
            fputc('L', stdout);
            break;
         case 'T':
            fputc('T', stdout);
            break;
      }
      fputc('>', stdout);
   }
   dumpTree((Run*) ptr->child);
   if (ptr->tag > 1) {
      fputc('<', stdout);
      fputc('/', stdout);
      fputc(ptr->tag, stdout);
      switch (ptr->tag) {
         case 'E':
            fputc('M', stdout);
            break;
         case 'P':
            fputc('L', stdout);
            break;
         case 'T':
            fputc('T', stdout);
            break;
      }
      fputc('>', stdout);
   }
   dumpTree(ptr->next);
}

void parse() {
   int ch;

   while ((ch = getchar()) != EOF) {
      if (!(attr & tt_flag) && isspace(ch)) {
         while (isspace(ch = getchar()));
         ungetc(ch, stdin);
         ch = ' ';
      }
      switch (ch) {
         case '<':
            if (text) {
               closeTag();
            }
            text = 0;
            tag();
            break;
         default:
            if (text) {
               append(ch);
            }
            else {
               newText(ch);
            }
            break;
      }
   }
   
   optimize();
   
   dumpTree(&root);
}



int main() {

   parse();

   return 0;
}

